@@ -1,6 +1,5 @@
 tv.taxval <- function (db, obs, refl, tv_home, concept, syn = c('adapt','conflict','preserve'), 
-subdiv = c('conflict', 'adapt', 'preserve'), ag = c('conflict', 'preserve', 'adapt'), mono = c('lower','higher', 'preserve'), monolist = "monotypic-D", 
-genus = c('delete','preserve'), quiet = FALSE, sysPath = FALSE, ...) 
+subdiv = c('conflict', 'adapt', 'preserve'), ag = c('conflict', 'adapt','preserve'), mono = c('lower','higher','all','preserve'), monolist = "monotypic-D", uncertain = NULL, genus = c('preserve','delete'), quiet = FALSE, sysPath = FALSE, ...) 
 {
 
     syn <- match.arg(syn)
@@ -21,7 +20,7 @@ genus = c('delete','preserve'), quiet = FALSE, sysPath = FALSE, ...)
     ### functions   
 adapt <- function(expr, mess, column, column2=c("SPECIES_NR", "ABBREVIAT", "Freq_Member","AGG", "AGG_NAME","Freq_Agg")) {    
     obsspec <- unique(obs$SPECIES_NR)
-    species$AGG_RANK <- species$RANK[match(species$AGG,species$SPECIES_NR)]
+    species$AGG_RANG <- species$RANG[match(species$AGG,species$SPECIES_NR)]
     temp <- species[eval(expr) & species$SPECIES_NR %in% obsspec,]
     if(column=='AGG') temp[,c('AGG','AGG_NAME')] <- species[match(temp$VALID_NR,species$SPECIES_NR),c('AGG','AGG_NAME')]
     vec <- temp[match(obs$SPECIES_NR, temp$SPECIES_NR), column]
@@ -37,7 +36,7 @@ adapt <- function(expr, mess, column, column2=c("SPECIES_NR", "ABBREVIAT", "Freq
     obs
     }
 
-confl <- function(expr, column, column2=c("SPECIES_NR", "ABBREVIAT", "Freq_Member","AGG", "AGG_NAME","Freq_Agg"), mess, comb='at species level') {
+confl <- function(expr, column, column2=c("SPECIES_NR", "ABBREVIAT", "Freq_Member","AGG", "AGG_NAME","Freq_Agg"), mess, comb='at higher level') {
     obsspec <- unique(obs$SPECIES_NR)
     tem <- species[eval(expr) & species$SPECIES_NR %in% obsspec, ]
     temp <- tem[tem[[column]] %in% obsspec, ]
@@ -58,6 +57,20 @@ preserve <- function(mess) {
     cat('\n', mess, 'preserved! \n')
     obs
     }
+
+unc <- function(obs, column, uncrow, i, ...) {
+  un <- match.arg(as.character(uncrow[[2]]),c('aggregate','preserve','ignore'))
+  if(un == 'aggregate') {
+    cat('\n changing species occurrences to coarser level for uncertainty level ', as.character(uncrow[[1]]))
+    sp <- obs$SPECIES_NR[obs[,column] == uncrow[[1]]]
+    taxa <- species[species$SPECIES_NR %in% sp,]  #tax(sp, refl=refl, tax=TRUE, sysPath=sysPath, ...)
+    taxa$AGG <- species$AGG[match(taxa$VALID_NR,species$SPECIES_NR)] 
+    for(n in 1:nrow(taxa)) obs$SPECIES_NR[obs$SPECIES_NR == taxa[n,'SPECIES_NR'] & obs[,column] == uncrow[[1]]] <- taxa[n,'AGG']
+   }
+  if(un %in% c('preserve','ignore')) {cat('\n preserving species occurrences of uncertainty level ',as.character(uncrow[[1]]))
+   }
+  obs
+ }
 ##########################
 
 fr <- as.data.frame(table(obs$SPECIES_NR))
@@ -71,8 +84,8 @@ obs <- switch(syn,
 fr <- as.data.frame(table(obs$SPECIES_NR))
    
 obs <- switch(subdiv,
-    conflict = confl(expr=expression(species$RANK %in% c('FOR','VAR','SGR','SSP','ZUS')), column='AGG', mess="Variants, forms, subspecies etc."),
-    adapt = adapt(expr=expression(species$RANK %in% c('FOR','VAR','SGR','SSP','ZUS')),  column='AGG', mess='Variants, forms, subspecies etc.'),
+    conflict = confl(expr=expression(species$RANG %in% c('FOR','VAR','SGR','SSP','ZUS')), column='AGG', mess="Variants, forms, subspecies etc."),
+    adapt = adapt(expr=expression(species$RANG %in% c('FOR','VAR','SGR','SSP','ZUS')),  column='AGG', mess='Variants, forms, subspecies etc.'),
     preserve = preserve('Variants, forms, subspecies etc.'),
   )
 
@@ -84,7 +97,7 @@ obs <- switch(ag,
     conflict = {
 	obsspec <- unique(obs$SPECIES_NR)
 	# Valid aggregates in Specieslist
-	AG <- species$SPECIES_NR[species$RANK %in% c('AGG','AG2','SEC','SGE','SER','SSE') & species$SYNONYM == FALSE]
+	AG <- species$SPECIES_NR[species$RANG %in% c('AGG','AG2','SEC','SGE','SER','SSE') & species$SYNONYM == FALSE]
 	# Obs-Species which are member of aggregates
 	agg.member <- species[species$AGG %in% AG & species$SPECIES_NR %in% obsspec, c("SPECIES_NR","ABBREVIAT", "AGG", "AGG_NAME")]
 	# agg.members which aggregates are also in dataset
@@ -98,44 +111,62 @@ obs <- switch(ag,
 	  obs$SPECIES_NR <- replace(obs$SPECIES_NR, which(!is.na(match(obs$SPECIES_NR, conf$SPECIES_NR))), conf$AGG[match(obs$SPECIES_NR, conf$SPECIES_NR)][!is.na(match(obs$SPECIES_NR, conf$SPECIES_NR))])
 	}
 	  obs     },
-      adapt = adapt(expr=expression(species$AGG_RANK == 'AGG'), column='AGG', column2=c("SPECIES_NR", "ABBREVIAT", "Freq_Member","AGG", "AGG_NAME", "Freq_Agg"), mess='Aggregates'),
+      adapt = adapt(expr=expression(species$AGG_RANG == 'AGG'), column='AGG', column2=c("SPECIES_NR", "ABBREVIAT", "Freq_Member","AGG", "AGG_NAME", "Freq_Agg"), mess='Aggregates'),
   ) #c('AGG','AG2','SEC','SGE','SER','SSE')
      
-if (mono %in% c("lower", "higher")) {
+if (mono %in% c("lower", "higher", "all")) {
     obsspec <- unique(obs$SPECIES_NR)
     if (file.access(paste(tv_home, 'Species', refl, paste(monolist, "dbf", sep = "."), sep = "/"))) 
         warning("List of monotypic taxa not available!") else Mono <- read.dbf(paste(tv_home, 'Species', refl, paste(monolist, "dbf", sep = "."), sep = "/"))
-    if (mono == "lower")    tmp <- Mono$MEMBER_NR[match(obs$SPECIES_NR, Mono$AGG_NR)]
-    if (mono == "higher")   tmp <- Mono$AGG_NR[match(obs$SPECIES_NR, Mono$MEMBER_NR)]
+    if (mono == "lower")  tmp <- Mono$MEMBER_NR[match(obs$SPECIES_NR, Mono$AGG_NR)]
+    if (mono == "all") 	tmp <- Mono$AGG_NR[match(obs$SPECIES_NR, Mono$MEMBER_NR)]
+    if (mono == "higher") {
+	  tmp <- Mono$AGG_NR[match(obs$SPECIES_NR, Mono$MEMBER_NR)]
+	  tmp[!Mono$MEMB_RANG[match(tmp, Mono$AGG_NR)] %in% c('FOR','VAR','ZUS','SSP')] <- NA
+      }
     if (sum(tmp > 0, na.rm = TRUE) > 0) {
       repeat{
         obs$SPECIES_NR <- replace(obs$SPECIES_NR, which(tmp > 0), tmp[!is.na(tmp)])
         cat(paste('\n',length(unique(tmp[!is.na(tmp)])), "Monotypic taxa found in dataset, species converted to", mono, "rank.",'\n'))
-        if (mono == "lower")    tmp <- Mono$MEMBER_NR[match(obs$SPECIES_NR, Mono$AGG_NR)]
-        if (mono == "higher")   tmp <- Mono$AGG_NR[match(obs$SPECIES_NR, Mono$MEMBER_NR)]
         if(!quiet) print(Mono[Mono$AGG_NR %in% obsspec, ])
+        if (mono == "lower")    tmp <- Mono$MEMBER_NR[match(obs$SPECIES_NR, Mono$AGG_NR)]
+	if (mono == "all") 	tmp <- Mono$AGG_NR[match(obs$SPECIES_NR, Mono$MEMBER_NR)]
+        if (mono == "higher") {
+	    tmp <- Mono$AGG_NR[match(obs$SPECIES_NR, Mono$MEMBER_NR)]
+	    tmp[!Mono$MEMB_RANG[match(tmp, Mono$AGG_NR)] %in% c('FOR','VAR','ZUS','SSP')] <- NA
+	    }
         if(sum(tmp > 0, na.rm = TRUE) == 0) break
         obsspec <- unique(obs$SPECIES_NR)
-      } } else cat('\n No monotypic taxa found. \n\n')
+      } } else if(mono=='higher') cat('\n No monotypic taxa below species level found. \n\n') else cat('\n No monotypic taxa found. \n\n')
   } else cat('\n Monotypic taxa preserved! \n\n')
 
-  ## Delete 
-  if(genus == 'delete') {
+  ## Uncertainty
+  if(!is.null(uncertain)) {
+    cat('\n Frequency of uncertainty levels')
+    print(table(obs[,uncertain[[1]]]))
+    species <- tax('all', refl = refl, tax=TRUE, tv_home=tv_home, sysPath=sysPath, ...)
+    for(i in 1:nrow(uncertain[[2]])) obs <- unc(obs, uncertain[[1]], uncertain[[2]][i,])
+    cat('\n')
+    }
+
+
+  ## Determined only to Aggregate level
     obsspec <- unique(obs$SPECIES_NR)
-    gat <- species[species$RANK %in% c('GAT','FAM', 'UKL', 'ORD', 'ABT', 'KLA', 'ROOT', 'UAB'), c("SPECIES_NR", "ABBREVIAT")]
+    gat <- species[species$RANG %in% c('GAT','FAM', 'UKL', 'ORD', 'ABT', 'KLA', 'ROOT', 'UAB'), c("SPECIES_NR", "ABBREVIAT")]
     if (any(gat$SPECIES_NR %in% obsspec)) {
-      cat('\n',sum(gat$SPECIES_NR %in% obsspec), "Undetermined genera or above found in dataset, deleted:",'\n')
-      if(!quiet) print(gat[gat$SPECIES_NR %in% obsspec, ])
-      } else cat('\n',"No undetermined genera or above found.",'\n')
-    obs <- obs[!obs$SPECIES_NR %in% gat$SPECIES_NR, ]
-  } else cat('\n Undetermined genera and above preserved! \n')
+      if(genus == 'delete') {
+	cat('\n',sum(gat$SPECIES_NR %in% obsspec), "Undetermined genera or above found in dataset, deleted:",'\n')
+	if(!quiet) print(gat[gat$SPECIES_NR %in% obsspec, ])
+	obs <- obs[!obs$SPECIES_NR %in% gat$SPECIES_NR, ]
+      } else cat('\n Undetermined genera and above preserved! \n')
+    } else cat('\n',"No undetermined genera or above found.",'\n')
     
 cat('\n Number of taxa after validation:', length(unique(obs$SPECIES_NR)),'\n\n')
 
 ### Critical species
 ## Pseudonyms
- auct <- species[grep("auct.", species$ABBREVIAT), c(1:5, 11, 13, 14, 15)]
-  auct$to_check <- sub(" auct.", "", auct$ABBREVIAT)
+ auct <- species[grep("\ auct.", species$ABBREVIAT, perl=TRUE), c(1:5, 11, 13, 14, 15)]
+  auct$to_check <- sub("\ auct.", "", auct$ABBREVIAT, perl=TRUE)
   auct$checknr <- species$SPECIES_NR[match(auct$to_check, species$ABBREVIAT)]
   auct <- auct[!is.na(auct$checknr), ]
   auct <- auct[, c(10, 11, 2, 1, 5, 4, 6)]
@@ -146,10 +177,10 @@ cat('\n Number of taxa after validation:', length(unique(obs$SPECIES_NR)),'\n\n'
       if(!quiet) print(auct[match(u, auct$checknr, nomatch = FALSE), ])
    }
 ### Extent of taxon interpretation
-  sl <- species[grep("s. l.", species$ABBREVIAT), c(1:5, 11, 13, 14, 15)]
-  sl$to_check <- sub(" s. l.", "", sl$ABBREVIAT)
-  sstr <- species[grep("s. str.", species$ABBREVIAT), c(1:5, 11, 13, 14, 15)]
-  sstr$to_check <- sub(" s. str.", "", sstr$ABBREVIAT)
+  sl <- species[grep("\ s.\ l.$", species$ABBREVIAT, perl=TRUE), c(1:5, 11, 13, 14, 15)]
+  sl$to_check <- sub("\ s.\ l.$", "", sl$ABBREVIAT, perl=TRUE)
+  sstr <- species[grep("\ s.\ str.$", species$ABBREVIAT, perl=TRUE), c(1:5, 11, 13, 14, 15)]
+  sstr$to_check <- sub("\ s.\ str.$", "", sstr$ABBREVIAT, perl=TRUE)
   ext <- rbind(sl,sstr)
 
   ext$checknr <- species$SPECIES_NR[match(ext$to_check, species$ABBREVIAT)]
